@@ -1,52 +1,66 @@
-"""RFC-822 message manipulation class.
+"""RFC 2822 message manipulation.
 
-XXX This is only a very rough sketch of a full RFC-822 parser;
-in particular the tokenizing of addresses does not adhere to all the
-quoting rules.
+Note: This is only a very rough sketch of a full RFC-822 parser; in particular
+the tokenizing of addresses does not adhere to all the quoting rules.
+
+Note: RFC 2822 is a long awaited update to RFC 822.  This module should
+conform to RFC 2822, and is thus mis-named (it's not worth renaming it).  Some
+effort at RFC 2822 updates have been made, but a thorough audit has not been
+performed.  Consider any RFC 2822 non-conformance to be a bug.
+
+    RFC 2822: http://www.faqs.org/rfcs/rfc2822.html
+    RFC 822 : http://www.faqs.org/rfcs/rfc822.html (obsolete)
 
 Directions for use:
 
 To create a Message object: first open a file, e.g.:
+
   fp = open(file, 'r')
+
 You can use any other legal way of getting an open file object, e.g. use
-sys.stdin or call os.popen().
-Then pass the open file object to the Message() constructor:
+sys.stdin or call os.popen().  Then pass the open file object to the Message()
+constructor:
+
   m = Message(fp)
 
-This class can work with any input object that supports a readline
-method.  If the input object has seek and tell capability, the
-rewindbody method will work; also illegal lines will be pushed back
-onto the input stream.  If the input object lacks seek but has an
-`unread' method that can push back a line of input, Message will use
-that to push back illegal lines.  Thus this class can be used to parse
-messages coming from a buffered stream.
+This class can work with any input object that supports a readline method.  If
+the input object has seek and tell capability, the rewindbody method will
+work; also illegal lines will be pushed back onto the input stream.  If the
+input object lacks seek but has an `unread' method that can push back a line
+of input, Message will use that to push back illegal lines.  Thus this class
+can be used to parse messages coming from a buffered stream.
 
-The optional `seekable' argument is provided as a workaround for
-certain stdio libraries in which tell() discards buffered data before
-discovering that the lseek() system call doesn't work.  For maximum
-portability, you should set the seekable argument to zero to prevent
-that initial \code{tell} when passing in an unseekable object such as
-a a file object created from a socket object.  If it is 1 on entry --
-which it is by default -- the tell() method of the open file object is
-called once; if this raises an exception, seekable is reset to 0.  For
-other nonzero values of seekable, this test is not made.
+The optional `seekable' argument is provided as a workaround for certain stdio
+libraries in which tell() discards buffered data before discovering that the
+lseek() system call doesn't work.  For maximum portability, you should set the
+seekable argument to zero to prevent that initial \code{tell} when passing in
+an unseekable object such as a file object created from a socket object.  If
+it is 1 on entry -- which it is by default -- the tell() method of the open
+file object is called once; if this raises an exception, seekable is reset to
+0.  For other nonzero values of seekable, this test is not made.
 
 To get the text of a particular header there are several methods:
+
   str = m.getheader(name)
   str = m.getrawheader(name)
-where name is the name of the header, e.g. 'Subject'.
-The difference is that getheader() strips the leading and trailing
-whitespace, while getrawheader() doesn't.  Both functions retain
-embedded whitespace (including newlines) exactly as they are
-specified in the header, and leave the case of the text unchanged.
+
+where name is the name of the header, e.g. 'Subject'.  The difference is that
+getheader() strips the leading and trailing whitespace, while getrawheader()
+doesn't.  Both functions retain embedded whitespace (including newlines)
+exactly as they are specified in the header, and leave the case of the text
+unchanged.
 
 For addresses and address lists there are functions
-  realname, mailaddress = m.getaddr(name) and
+
+  realname, mailaddress = m.getaddr(name)
   list = m.getaddrlist(name)
+
 where the latter returns a list of (realname, mailaddr) tuples.
 
 There is also a method
+
   time = m.getdate(name)
+
 which parses a Date-like field and returns a time-compatible tuple,
 i.e. a tuple such as returned by time.localtime() or accepted by
 time.mktime().
@@ -59,13 +73,17 @@ There are also some utility functions here.
 
 import time
 
+from warnings import warnpy3k
+warnpy3k("in 3.x, rfc822 has been removed in favor of the email package",
+         stacklevel=2)
+
 __all__ = ["Message","AddressList","parsedate","parsedate_tz","mktime_tz"]
 
 _blanklines = ('\r\n', '\n')            # Optimization for islast()
 
 
 class Message:
-    """Represents a single RFC-822-compliant message."""
+    """Represents a single RFC 2822-compliant message."""
 
     def __init__(self, fp, seekable = 1):
         """Initialize the class instance and read the headers."""
@@ -74,10 +92,8 @@ class Message:
             # (and then assume seek() works, too)
             try:
                 fp.tell()
-            except:
+            except (AttributeError, IOError):
                 seekable = 0
-            else:
-                seekable = 1
         self.fp = fp
         self.seekable = seekable
         self.startofheaders = None
@@ -106,22 +122,21 @@ class Message:
     def readheaders(self):
         """Read header lines.
 
-        Read header lines up to the entirely blank line that
-        terminates them.  The (normally blank) line that ends the
-        headers is skipped, but not included in the returned list.
-        If a non-header line ends the headers, (which is an error),
-        an attempt is made to backspace over it; it is never
-        included in the returned list.
+        Read header lines up to the entirely blank line that terminates them.
+        The (normally blank) line that ends the headers is skipped, but not
+        included in the returned list.  If a non-header line ends the headers,
+        (which is an error), an attempt is made to backspace over it; it is
+        never included in the returned list.
 
-        The variable self.status is set to the empty string if all
-        went well, otherwise it is an error message.
-        The variable self.headers is a completely uninterpreted list
-        of lines contained in the header (so printing them will
-        reproduce the header exactly as it appears in the file).
+        The variable self.status is set to the empty string if all went well,
+        otherwise it is an error message.  The variable self.headers is a
+        completely uninterpreted list of lines contained in the header (so
+        printing them will reproduce the header exactly as it appears in the
+        file).
         """
         self.dict = {}
         self.unixfrom = ''
-        self.headers = list = []
+        self.headers = lst = []
         self.status = ''
         headerseen = ""
         firstline = 1
@@ -148,7 +163,7 @@ class Message:
             firstline = 0
             if headerseen and line[0] in ' \t':
                 # It's a continuation line.
-                list.append(line)
+                lst.append(line)
                 x = (self.dict[headerseen] + "\n " + line.strip())
                 self.dict[headerseen] = x.strip()
                 continue
@@ -161,9 +176,14 @@ class Message:
             headerseen = self.isheader(line)
             if headerseen:
                 # It's a legal header line, save it.
-                list.append(line)
+                lst.append(line)
                 self.dict[headerseen] = line[len(headerseen)+1:].strip()
                 continue
+            elif headerseen is not None:
+                # An empty header name. These aren't allowed in HTTP, but it's
+                # probably a benign mistake. Don't add the header, just keep
+                # going.
+                continue
             else:
                 # It's not a header line; throw it back and stop here.
                 if not self.dict:
@@ -183,49 +203,45 @@ class Message:
         """Determine whether a given line is a legal header.
 
         This method should return the header name, suitably canonicalized.
-        You may override this method in order to use Message parsing
-        on tagged data in RFC822-like formats with special header formats.
+        You may override this method in order to use Message parsing on tagged
+        data in RFC 2822-like formats with special header formats.
         """
         i = line.find(':')
-        if i > 0:
+        if i > -1:
             return line[:i].lower()
-        else:
-            return None
+        return None
 
     def islast(self, line):
-        """Determine whether a line is a legal end of RFC-822 headers.
+        """Determine whether a line is a legal end of RFC 2822 headers.
 
-        You may override this method if your application wants
-        to bend the rules, e.g. to strip trailing whitespace,
-        or to recognize MH template separators ('--------').
-        For convenience (e.g. for code reading from sockets) a
-        line consisting of \r\n also matches.
+        You may override this method if your application wants to bend the
+        rules, e.g. to strip trailing whitespace, or to recognize MH template
+        separators ('--------').  For convenience (e.g. for code reading from
+        sockets) a line consisting of \\r\\n also matches.
         """
         return line in _blanklines
 
     def iscomment(self, line):
         """Determine whether a line should be skipped entirely.
 
-        You may override this method in order to use Message parsing
-        on tagged data in RFC822-like formats that support embedded
-        comments or free-text data.
+        You may override this method in order to use Message parsing on tagged
+        data in RFC 2822-like formats that support embedded comments or
+        free-text data.
         """
-        return None
+        return False
 
     def getallmatchingheaders(self, name):
         """Find all header lines matching a given header name.
 
-        Look through the list of headers and find all lines
-        matching a given header name (and their continuation
-        lines).  A list of the lines is returned, without
-        interpretation.  If the header does not occur, an
-        empty list is returned.  If the header occurs multiple
-        times, all occurrences are returned.  Case is not
-        important in the header name.
+        Look through the list of headers and find all lines matching a given
+        header name (and their continuation lines).  A list of the lines is
+        returned, without interpretation.  If the header does not occur, an
+        empty list is returned.  If the header occurs multiple times, all
+        occurrences are returned.  Case is not important in the header name.
         """
         name = name.lower() + ':'
         n = len(name)
-        list = []
+        lst = []
         hit = 0
         for line in self.headers:
             if line[:n].lower() == name:
@@ -233,19 +249,18 @@ class Message:
             elif not line[:1].isspace():
                 hit = 0
             if hit:
-                list.append(line)
-        return list
+                lst.append(line)
+        return lst
 
     def getfirstmatchingheader(self, name):
         """Get the first header line matching name.
 
-        This is similar to getallmatchingheaders, but it returns
-        only the first matching header (and its continuation
-        lines).
+        This is similar to getallmatchingheaders, but it returns only the
+        first matching header (and its continuation lines).
         """
         name = name.lower() + ':'
         n = len(name)
-        list = []
+        lst = []
         hit = 0
         for line in self.headers:
             if hit:
@@ -254,46 +269,40 @@ class Message:
             elif line[:n].lower() == name:
                 hit = 1
             if hit:
-                list.append(line)
-        return list
+                lst.append(line)
+        return lst
 
     def getrawheader(self, name):
         """A higher-level interface to getfirstmatchingheader().
 
-        Return a string containing the literal text of the
-        header but with the keyword stripped.  All leading,
-        trailing and embedded whitespace is kept in the
-        string, however.
-        Return None if the header does not occur.
+        Return a string containing the literal text of the header but with the
+        keyword stripped.  All leading, trailing and embedded whitespace is
+        kept in the string, however.  Return None if the header does not
+        occur.
         """
 
-        list = self.getfirstmatchingheader(name)
-        if not list:
+        lst = self.getfirstmatchingheader(name)
+        if not lst:
             return None
-        list[0] = list[0][len(name) + 1:]
-        return ''.join(list)
+        lst[0] = lst[0][len(name) + 1:]
+        return ''.join(lst)
 
     def getheader(self, name, default=None):
         """Get the header value for a name.
 
-        This is the normal interface: it returns a stripped
-        version of the header value for a given header name,
-        or None if it doesn't exist.  This uses the dictionary
-        version which finds the *last* such header.
+        This is the normal interface: it returns a stripped version of the
+        header value for a given header name, or None if it doesn't exist.
+        This uses the dictionary version which finds the *last* such header.
         """
-        try:
-            return self.dict[name.lower()]
-        except KeyError:
-            return default
+        return self.dict.get(name.lower(), default)
     get = getheader
 
     def getheaders(self, name):
         """Get all values for a header.
 
-        This returns a list of values for headers given more than once;
-        each value in the result list is stripped in the same way as the
-        result of getheader().  If the header is not given, return an
-        empty list.
+        This returns a list of values for headers given more than once; each
+        value in the result list is stripped in the same way as the result of
+        getheader().  If the header is not given, return an empty list.
         """
         result = []
         current = ''
@@ -332,7 +341,6 @@ class Message:
         Retrieves a list of addresses from a header, where each address is a
         tuple as returned by getaddr().  Scans all named headers, so it works
         properly with multiple To: or Cc: headers for example.
-
         """
         raw = []
         for h in self.getallmatchingheaders(name):
@@ -346,14 +354,14 @@ class Message:
                     addr = h[i+1:]
                 raw.append(addr)
         alladdrs = ''.join(raw)
-        a = AddrlistClass(alladdrs)
-        return a.getaddrlist()
+        a = AddressList(alladdrs)
+        return a.addresslist
 
     def getdate(self, name):
         """Retrieve a date field from a header.
 
-        Retrieves a date field from the named header, returning
-        a tuple compatible with time.mktime().
+        Retrieves a date field from the named header, returning a tuple
+        compatible with time.mktime().
         """
         try:
             data = self[name]
@@ -364,9 +372,8 @@ class Message:
     def getdate_tz(self, name):
         """Retrieve a date field from a header as a 10-tuple.
 
-        The first 9 elements make up a tuple compatible with
-        time.mktime(), and the 10th is the offset of the poster's
-        time zone from GMT/UTC.
+        The first 9 elements make up a tuple compatible with time.mktime(),
+        and the 10th is the offset of the poster's time zone from GMT/UTC.
         """
         try:
             data = self[name]
@@ -388,26 +395,25 @@ class Message:
     def __setitem__(self, name, value):
         """Set the value of a header.
 
-        Note: This is not a perfect inversion of __getitem__, because
-        any changed headers get stuck at the end of the raw-headers list
-        rather than where the altered header was.
+        Note: This is not a perfect inversion of __getitem__, because any
+        changed headers get stuck at the end of the raw-headers list rather
+        than where the altered header was.
         """
         del self[name] # Won't fail if it doesn't exist
         self.dict[name.lower()] = value
         text = name + ": " + value
-        lines = text.split("\n")
-        for line in lines:
+        for line in text.split("\n"):
             self.headers.append(line + "\n")
 
     def __delitem__(self, name):
         """Delete all occurrences of a specific header, if it is present."""
         name = name.lower()
-        if not self.dict.has_key(name):
+        if not name in self.dict:
             return
         del self.dict[name]
         name = name + ':'
         n = len(name)
-        list = []
+        lst = []
         hit = 0
         for i in range(len(self.headers)):
             line = self.headers[i]
@@ -416,14 +422,31 @@ class Message:
             elif not line[:1].isspace():
                 hit = 0
             if hit:
-                list.append(i)
-        list.reverse()
-        for i in list:
+                lst.append(i)
+        for i in reversed(lst):
             del self.headers[i]
 
+    def setdefault(self, name, default=""):
+        lowername = name.lower()
+        if lowername in self.dict:
+            return self.dict[lowername]
+        else:
+            text = name + ": " + default
+            for line in text.split("\n"):
+                self.headers.append(line + "\n")
+            self.dict[lowername] = default
+            return default
+
     def has_key(self, name):
         """Determine whether a message contains the named header."""
-        return self.dict.has_key(name.lower())
+        return name.lower() in self.dict
+
+    def __contains__(self, name):
+        """Determine whether a message contains the named header."""
+        return name.lower() in self.dict
+
+    def __iter__(self):
+        return iter(self.dict)
 
     def keys(self):
         """Get all of a message's header field names."""
@@ -441,10 +464,7 @@ class Message:
         return self.dict.items()
 
     def __str__(self):
-        str = ''
-        for hdr in self.headers:
-            str = str + hdr
-        return str
+        return ''.join(self.headers)
 
 
 # Utility functions
@@ -454,36 +474,37 @@ class Message:
 # XXX The inverses of the parse functions may also be useful.
 
 
-def unquote(str):
+def unquote(s):
     """Remove quotes from a string."""
-    if len(str) > 1:
-        if str[0] == '"' and str[-1:] == '"':
-            return str[1:-1]
-        if str[0] == '<' and str[-1:] == '>':
-            return str[1:-1]
-    return str
+    if len(s) > 1:
+        if s.startswith('"') and s.endswith('"'):
+            return s[1:-1].replace('\\\\', '\\').replace('\\"', '"')
+        if s.startswith('<') and s.endswith('>'):
+            return s[1:-1]
+    return s
 
 
-def quote(str):
+def quote(s):
     """Add quotes around a string."""
-    return str.replace('\\', '\\\\').replace('"', '\\"')
+    return s.replace('\\', '\\\\').replace('"', '\\"')
 
 
 def parseaddr(address):
     """Parse an address into a (realname, mailaddr) tuple."""
-    a = AddrlistClass(address)
-    list = a.getaddrlist()
-    if not list:
+    a = AddressList(address)
+    lst = a.addresslist
+    if not lst:
         return (None, None)
-    else:
-        return list[0]
+    return lst[0]
 
 
 class AddrlistClass:
     """Address parser class by Ben Escoto.
 
     To understand what this class does, it helps to have a copy of
-    RFC-822 in front of you.
+    RFC 2822 in front of you.
+
+    http://www.faqs.org/rfcs/rfc2822.html
 
     Note: this class interface is deprecated and may be removed in the future.
     Use rfc822.AddressList instead.
@@ -492,14 +513,18 @@ class AddrlistClass:
     def __init__(self, field):
         """Initialize a new instance.
 
-        `field' is an unparsed address header field, containing
-        one or more addresses.
+        `field' is an unparsed address header field, containing one or more
+        addresses.
         """
         self.specials = '()<>@,:;.\"[]'
         self.pos = 0
         self.LWS = ' \t'
         self.CR = '\r\n'
         self.atomends = self.specials + self.LWS + self.CR
+        # Note that RFC 2822 now specifies `.' as obs-phrase, meaning that it
+        # is obsolete syntax.  RFC 2822 requires that we recognize obsolete
+        # syntax, so allow dots in phrases.
+        self.phraseends = self.atomends.replace('.', '')
         self.field = field
         self.commentlist = []
 
@@ -517,10 +542,12 @@ class AddrlistClass:
 
         Returns a list containing all of the addresses.
         """
+        result = []
         ad = self.getaddress()
-        if ad:
-            return ad + self.getaddrlist()
-        else: return []
+        while ad:
+            result += ad
+            ad = self.getaddress()
+        return result
 
     def getaddress(self):
         """Parse the next address."""
@@ -552,11 +579,11 @@ class AddrlistClass:
             returnlist = []
 
             fieldlen = len(self.field)
-            self.pos = self.pos + 1
+            self.pos += 1
             while self.pos < len(self.field):
                 self.gotonext()
                 if self.pos < fieldlen and self.field[self.pos] == ';':
-                    self.pos = self.pos + 1
+                    self.pos += 1
                     break
                 returnlist = returnlist + self.getaddress()
 
@@ -573,11 +600,11 @@ class AddrlistClass:
             if plist:
                 returnlist = [(' '.join(self.commentlist), plist[0])]
             elif self.field[self.pos] in self.specials:
-                self.pos = self.pos + 1
+                self.pos += 1
 
         self.gotonext()
         if self.pos < len(self.field) and self.field[self.pos] == ',':
-            self.pos = self.pos + 1
+            self.pos += 1
         return returnlist
 
     def getrouteaddr(self):
@@ -589,7 +616,7 @@ class AddrlistClass:
             return
 
         expectroute = 0
-        self.pos = self.pos + 1
+        self.pos += 1
         self.gotonext()
         adlist = ""
         while self.pos < len(self.field):
@@ -597,31 +624,30 @@ class AddrlistClass:
                 self.getdomain()
                 expectroute = 0
             elif self.field[self.pos] == '>':
-                self.pos = self.pos + 1
+                self.pos += 1
                 break
             elif self.field[self.pos] == '@':
-                self.pos = self.pos + 1
+                self.pos += 1
                 expectroute = 1
             elif self.field[self.pos] == ':':
-                self.pos = self.pos + 1
-                expectaddrspec = 1
+                self.pos += 1
             else:
                 adlist = self.getaddrspec()
-                self.pos = self.pos + 1
+                self.pos += 1
                 break
             self.gotonext()
 
         return adlist
 
     def getaddrspec(self):
-        """Parse an RFC-822 addr-spec."""
+        """Parse an RFC 2822 addr-spec."""
         aslist = []
 
         self.gotonext()
         while self.pos < len(self.field):
             if self.field[self.pos] == '.':
                 aslist.append('.')
-                self.pos = self.pos + 1
+                self.pos += 1
             elif self.field[self.pos] == '"':
                 aslist.append('"%s"' % self.getquote())
             elif self.field[self.pos] in self.atomends:
@@ -633,7 +659,7 @@ class AddrlistClass:
             return ''.join(aslist)
 
         aslist.append('@')
-        self.pos = self.pos + 1
+        self.pos += 1
         self.gotonext()
         return ''.join(aslist) + self.getdomain()
 
@@ -642,13 +668,13 @@ class AddrlistClass:
         sdlist = []
         while self.pos < len(self.field):
             if self.field[self.pos] in self.LWS:
-                self.pos = self.pos + 1
+                self.pos += 1
             elif self.field[self.pos] == '(':
                 self.commentlist.append(self.getcomment())
             elif self.field[self.pos] == '[':
                 sdlist.append(self.getdomainliteral())
             elif self.field[self.pos] == '.':
-                self.pos = self.pos + 1
+                self.pos += 1
                 sdlist.append('.')
             elif self.field[self.pos] in self.atomends:
                 break
@@ -658,36 +684,37 @@ class AddrlistClass:
     def getdelimited(self, beginchar, endchars, allowcomments = 1):
         """Parse a header fragment delimited by special characters.
 
-        `beginchar' is the start character for the fragment.
-        If self is not looking at an instance of `beginchar' then
-        getdelimited returns the empty string.
+        `beginchar' is the start character for the fragment.  If self is not
+        looking at an instance of `beginchar' then getdelimited returns the
+        empty string.
 
         `endchars' is a sequence of allowable end-delimiting characters.
         Parsing stops when one of these is encountered.
 
-        If `allowcomments' is non-zero, embedded RFC-822 comments
-        are allowed within the parsed fragment.
+        If `allowcomments' is non-zero, embedded RFC 2822 comments are allowed
+        within the parsed fragment.
         """
         if self.field[self.pos] != beginchar:
             return ''
 
         slist = ['']
         quote = 0
-        self.pos = self.pos + 1
+        self.pos += 1
         while self.pos < len(self.field):
             if quote == 1:
                 slist.append(self.field[self.pos])
                 quote = 0
             elif self.field[self.pos] in endchars:
-                self.pos = self.pos + 1
+                self.pos += 1
                 break
             elif allowcomments and self.field[self.pos] == '(':
                 slist.append(self.getcomment())
+                continue        # have already advanced pos from getcomment
             elif self.field[self.pos] == '\\':
                 quote = 1
             else:
                 slist.append(self.field[self.pos])
-            self.pos = self.pos + 1
+            self.pos += 1
 
         return ''.join(slist)
 
@@ -700,45 +727,53 @@ class AddrlistClass:
         return self.getdelimited('(', ')\r', 1)
 
     def getdomainliteral(self):
-        """Parse an RFC-822 domain-literal."""
+        """Parse an RFC 2822 domain-literal."""
         return '[%s]' % self.getdelimited('[', ']\r', 0)
 
-    def getatom(self):
-        """Parse an RFC-822 atom."""
+    def getatom(self, atomends=None):
+        """Parse an RFC 2822 atom.
+
+        Optional atomends specifies a different set of end token delimiters
+        (the default is to use self.atomends).  This is used e.g. in
+        getphraselist() since phrase endings must not include the `.' (which
+        is legal in phrases)."""
         atomlist = ['']
+        if atomends is None:
+            atomends = self.atomends
 
         while self.pos < len(self.field):
-            if self.field[self.pos] in self.atomends:
+            if self.field[self.pos] in atomends:
                 break
             else: atomlist.append(self.field[self.pos])
-            self.pos = self.pos + 1
+            self.pos += 1
 
         return ''.join(atomlist)
 
     def getphraselist(self):
-        """Parse a sequence of RFC-822 phrases.
+        """Parse a sequence of RFC 2822 phrases.
 
-        A phrase is a sequence of words, which are in turn either
-        RFC-822 atoms or quoted-strings.  Phrases are canonicalized
-        by squeezing all runs of continuous whitespace into one space.
+        A phrase is a sequence of words, which are in turn either RFC 2822
+        atoms or quoted-strings.  Phrases are canonicalized by squeezing all
+        runs of continuous whitespace into one space.
         """
         plist = []
 
         while self.pos < len(self.field):
             if self.field[self.pos] in self.LWS:
-                self.pos = self.pos + 1
+                self.pos += 1
             elif self.field[self.pos] == '"':
                 plist.append(self.getquote())
             elif self.field[self.pos] == '(':
                 self.commentlist.append(self.getcomment())
-            elif self.field[self.pos] in self.atomends:
+            elif self.field[self.pos] in self.phraseends:
                 break
-            else: plist.append(self.getatom())
+            else:
+                plist.append(self.getatom(self.phraseends))
 
         return plist
 
 class AddressList(AddrlistClass):
-    """An AddressList encapsulates a list of parsed RFC822 addresses."""
+    """An AddressList encapsulates a list of parsed RFC 2822 addresses."""
     def __init__(self, field):
         AddrlistClass.__init__(self, field)
         if field:
@@ -822,10 +857,17 @@ def parsedate_tz(data):
 
     Accounts for military timezones.
     """
+    if not data:
+        return None
     data = data.split()
     if data[0][-1] in (',', '.') or data[0].lower() in _daynames:
         # There's a dayname here. Skip it
         del data[0]
+    else:
+        # no space after the "weekday,"?
+        i = data[0].rfind(',')
+        if i >= 0:
+            data[0] = data[0][i+1:]
     if len(data) == 3: # RFC 850 date, deprecated
         stuff = data[0].split('-')
         if len(stuff) == 3:
@@ -877,7 +919,7 @@ def parsedate_tz(data):
         return None
     tzoffset = None
     tz = tz.upper()
-    if _timezones.has_key(tz):
+    if tz in _timezones:
         tzoffset = _timezones[tz]
     else:
         try:
@@ -891,17 +933,16 @@ def parsedate_tz(data):
             tzoffset = -tzoffset
         else:
             tzsign = 1
-        tzoffset = tzsign * ( (tzoffset/100)*3600 + (tzoffset % 100)*60)
-    tuple = (yy, mm, dd, thh, tmm, tss, 0, 0, 0, tzoffset)
-    return tuple
+        tzoffset = tzsign * ( (tzoffset//100)*3600 + (tzoffset % 100)*60)
+    return (yy, mm, dd, thh, tmm, tss, 0, 1, 0, tzoffset)
 
 
 def parsedate(data):
     """Convert a time string to a time tuple."""
     t = parsedate_tz(data)
-    if type(t) == type( () ):
-        return t[:9]
-    else: return t
+    if t is None:
+        return t
+    return t[:9]
 
 
 def mktime_tz(data):
@@ -917,11 +958,21 @@ def formatdate(timeval=None):
     """Returns time format preferred for Internet standards.
 
     Sun, 06 Nov 1994 08:49:37 GMT  ; RFC 822, updated by RFC 1123
+
+    According to RFC 1123, day and month names must always be in
+    English.  If not for that, this code could use strftime().  It
+    can't because strftime() honors the locale and could generated
+    non-English names.
     """
     if timeval is None:
         timeval = time.time()
-    return "%s" % time.strftime('%a, %d %b %Y %H:%M:%S GMT',
-                                time.gmtime(timeval))
+    timeval = time.gmtime(timeval)
+    return "%s, %02d %s %04d %02d:%02d:%02d GMT" % (
+            ("Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun")[timeval[6]],
+            timeval[2],
+            ("Jan", "Feb", "Mar", "Apr", "May", "Jun",
+             "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")[timeval[1]-1],
+                                timeval[0], timeval[3], timeval[4], timeval[5])
 
 
 # When used as script, run a small test program.
@@ -954,12 +1005,12 @@ if __name__ == '__main__':
     m.rewindbody()
     n = 0
     while f.readline():
-        n = n + 1
+        n += 1
     print 'Lines:', n
     print '-'*70
     print 'len =', len(m)
-    if m.has_key('Date'): print 'Date =', m['Date']
-    if m.has_key('X-Nonsense'): pass
+    if 'Date' in m: print 'Date =', m['Date']
+    if 'X-Nonsense' in m: pass
     print 'keys =', m.keys()
     print 'values =', m.values()
     print 'items =', m.items()
